@@ -46,6 +46,30 @@
 
 
 
+/**
+ * xfce_gdk_monitor_get_number:
+ * @monitor : a #GdkMonitor.
+ *
+ * Returns the index of the @monitor in its display.
+ *
+ * Return value: index of the @monitor
+ **/
+static gint
+xfce_gdk_monitor_get_number (GdkMonitor *monitor)
+{
+  GdkDisplay *display;
+  int num_monitors;
+
+  display = gdk_monitor_get_display (monitor);
+  num_monitors = gdk_display_get_n_monitors (display);
+  for (int i = 0; i < num_monitors; i++)
+    if (gdk_display_get_monitor (display, i) == monitor)
+      return i;
+
+  return -1;
+}
+
+
 /**
  * xfce_gdk_screen_get_active:
  * @monitor_return : (out) (allow-none): Address to store the monitor number to or %NULL.
@@ -60,26 +84,15 @@ GdkScreen *
 xfce_gdk_screen_get_active (gint *monitor_return)
 {
   GdkDisplay       *display;
+  GdkMonitor       *monitor;
   gint              rootx, rooty;
   GdkScreen        *screen;
 
-#if 0 /* GTK_CHECK_VERSION (3, 20, 0) */
   GdkSeat *seat;
 
   display = gdk_display_get_default ();
   seat = gdk_display_get_default_seat (display);
   gdk_device_get_position (gdk_seat_get_pointer (seat), &screen, &rootx, &rooty);
-#elif GTK_CHECK_VERSION (3, 0, 0)
-  GdkDeviceManager *manager;
-
-  display = gdk_display_get_default ();
-  manager = gdk_display_get_device_manager (display);
-  gdk_device_get_position (gdk_device_manager_get_client_pointer (manager),
-                           &screen, &rootx, &rooty);
-#else
-  display = gdk_display_get_default ();
-  gdk_display_get_pointer (display, &screen, &rootx, &rooty, NULL);
-#endif
 
   if (G_UNLIKELY (screen == NULL))
     {
@@ -91,7 +104,10 @@ xfce_gdk_screen_get_active (gint *monitor_return)
     {
       /* return the monitor number */
       if (monitor_return != NULL)
-        *monitor_return = gdk_screen_get_monitor_at_point (screen, rootx, rooty);
+        {
+          monitor = gdk_display_get_monitor_at_point (display, rootx, rooty);
+          *monitor_return = xfce_gdk_monitor_get_number (monitor);
+        }
     }
 
   return screen;
@@ -113,8 +129,6 @@ GdkRectangle *
 xfce_gdk_screen_get_geometry (void)
 {
   GdkRectangle *geometry;
-
-#if GTK_CHECK_VERSION (3, 22, 0)
   gint x, y, w, h;
   int num_monitors;
   GdkDisplay *display;
@@ -142,16 +156,54 @@ xfce_gdk_screen_get_geometry (void)
 
   geometry->width = w - x;
   geometry->height = h - y;
-#else
-  geometry = g_new0 (GdkRectangle, 1);
-
-  geometry->width = gdk_screen_width ();
-  geometry->height = gdk_screen_height ();
-#endif
 
   return geometry;
 }
 
 
+
+/**
+ * xfce_gdk_device_grab:
+ * @seat : A #GdkSeat.
+ * @window : The #GdkWindow which will own the grab.
+ * @capabilities : Capabilities that will be grabbed.
+ * @cursor : (nullable): The cursor to display while the grab is active. If this
+ *                      is %NULL then the normal cursors are used for window and
+ *                      its descendants, and the cursor for window is used
+ *                      elsewhere.
+ *
+ * Similar to gdk_seat_grab but tries to grab the seat five times with 100ms
+ * between each attempt.
+ *
+ * Return value: %TRUE on success, %FALSE otherwise.
+ *
+ * Since: 4.18
+ **/
+gboolean
+xfce_gdk_device_grab (GdkSeat            *seat,
+                      GdkWindow          *window,
+                      GdkSeatCapabilities capabilities,
+                      GdkCursor          *cursor)
+{
+  GdkGrabStatus status;
+  gint attempts = 0;
+
+  while (TRUE) {
+    status = gdk_seat_grab (seat, window, capabilities, FALSE, cursor, NULL,
+                            NULL, NULL);
+
+    if (status == GDK_GRAB_SUCCESS)
+      return TRUE;
+    if (attempts++ >= 5)
+      return FALSE;
+
+    /* Wait 100ms before trying again, useful when invoked by global hotkey
+     * because xfsettings will grab the key for a moment */
+    g_usleep(100000);
+  }
+
+  return FALSE;
+}
+
 #define __XFCE_GDK_EXTENSIONS_C__
 #include <libxfce4ui/libxfce4ui-aliasdef.c>
