@@ -355,7 +355,7 @@ xfce_gtk_toggle_menu_item_new_from_action_entry (const XfceGtkActionEntry *actio
  * xfce_gtk_tool_button_new_from_action_entry:
  * @action_entry : Label to use for the #GtkToolButton
  * @callback_param : optional callback parameter, or %NULL.
- * @toolbar_to_append_item : #GtkToolBar on which the item should be appended
+ * @toolbar_to_append_item : #GtkToolbar on which the item should be appended
  *
  * Method to create a toolbar button from the passed action entry.
  *
@@ -388,7 +388,7 @@ xfce_gtk_tool_button_new_from_action_entry (const XfceGtkActionEntry *action_ent
  * @action_entry : Label to use for the #GtkToggleToolButton
  * @callback_param : optional callback parameter, or %NULL.
  * @active : boolean value indicating whether the toggle is initially active.
- * @toolbar_to_append_item : #GtkToolBar on which the item should be appended
+ * @toolbar_to_append_item : #GtkToolbar on which the item should be appended
  *
  * Method to create a toolbar toggle-button from the passed action entry.
  *
@@ -974,7 +974,7 @@ xfce_widget_reparent (GtkWidget *widget,
 gchar *
 xfce_icon_name_from_desktop_id (const gchar *desktop_id)
 {
-    gchar *icon_file;
+    gchar *icon_file = NULL;
     gchar *resource;
     XfceRc *rcfile;
 
@@ -985,16 +985,16 @@ xfce_icon_name_from_desktop_id (const gchar *desktop_id)
                                   resource, TRUE);
     g_free (resource);
 
-    if (rcfile && xfce_rc_has_group (rcfile, "Desktop Entry")) {
-        xfce_rc_set_group (rcfile, "Desktop Entry");
-        icon_file = g_strdup (xfce_rc_read_entry (rcfile, "Icon", NULL));
+    if (rcfile != NULL) {
+        if (xfce_rc_has_group (rcfile, "Desktop Entry")) {
+            xfce_rc_set_group (rcfile, "Desktop Entry");
+            icon_file = g_strdup (xfce_rc_read_entry (rcfile, "Icon", NULL));
+        }
 
         xfce_rc_close (rcfile);
-
-        return icon_file;
     }
-    else
-        return NULL;
+
+    return icon_file;
 }
 
 
@@ -1052,8 +1052,13 @@ xfce_gicon_from_name (const gchar *name)
                                                     GTK_ICON_SIZE_BUTTON,
                                                     GTK_ICON_LOOKUP_FORCE_REGULAR);
 
-        if (icon_info)
+        if (icon_info) {
+            g_object_unref (icon_info);
             return gicon;
+        }
+        else {
+            g_object_unref (gicon);
+        }
     }
 
     return NULL;
@@ -1170,7 +1175,7 @@ xfce_has_gtk_frame_extents (GdkWindow *window,
  * @label  : a #GtkLabel.
  * @widget : a #GtkWidget.
  *
- * Sets the %ATK_RELATION_LABEL_FOR relation on @label for @widget, which means
+ * Sets the `ATK_RELATION_LABEL_FOR` relation on @label for @widget, which means
  * accessiblity tools will identify @label as descriptive item for the specified
  * @widget.
  **/
