@@ -66,6 +66,7 @@ static const gchar *const arch_64[] = {
   "aarch64",
   "amd64",
   "ia64",
+  "loongarch64",
   "ppc64",
   "sparc64",
   "x86_64",
@@ -198,7 +199,7 @@ get_cpu_info (const glibtop_sysinfo *info)
   /* count duplicates */
   for (i = 0; i != info->ncpu; ++i)
     {
-      const char * const keys[] = { "model name", "cpu", "Processor" };
+      const char * const keys[] = { "model name", "cpu", "Model Name", "Processor" };
       char *model;
       int  *count;
 
@@ -681,6 +682,9 @@ get_system_info (guint infotype)
       case DEVICE_NAME:
         result = g_strdup (buffer.nodename);
         break;
+      case KERNEL:
+        result = g_strdup (buffer.release);
+        break;
     }
 
   return result;
